/*--------------------------------------------------------------------------*/
/*--------------------------- File DCRwriter.cpp ---------------------------*/
/*--------------------------------------------------------------------------*/
/*--------------------------- IMPLEMENTATION -------------------------------*/
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/*------------------------------ INCLUDES ----------------------------------*/
/*--------------------------------------------------------------------------*/

#include "DCRwriter.h"

#include <string.h>
#include <fstream>

/*--------------------------------------------------------------------------*/
/*----------------------- IMPLEMENTATION OF DCRwriter  ---------------------*/
/*--------------------------------------------------------------------------*/
/*--------------------------- PUBLIC METHODS -------------------------------*/
/*--------------------------------------------------------------------------*/
/*----------------------------- CONSTRUCTOR --------------------------------*/
/*--------------------------------------------------------------------------*/

DCRwriter::DCRwriter( char *Name_topo , char *Name_traf , char* filename ,
		      char ftype) 
{
  // Import topology from file
  topology = fnss::Parser::parseTopology(Name_topo);

  // Import traffic-matrix from file
  traffic_matrix_sequence = fnss::Parser::parseTrafficMatrixSequence(Name_traf);
  traffic_matrix = traffic_matrix_sequence.getMatrix(0);

  // get list of all edges
  edges = topology.getAllEdges();

  // get list of all nodes
  nodes = topology.getAllNodes();

  // get list of all flows
  flows = traffic_matrix.getPairs();

  //get #nodes, #arcs, #commodities
  nnodes = topology.nodeCount();

  if(topology.isDirected())
    nlinks = topology.edgeCount();
  else
    nlinks = 2*topology.edgeCount();

  nflows = traffic_matrix.size();

  #ifdef FLOWSLIMIT
   if((log(nnodes)/log(2))*nnodes < nflows)
     nflows = (log(nnodes)/log(2))*nnodes;
   cout << "\n nlog flows is: " << nflows;
  #endif

  try {
   Flows = new DCR::DCRFlow[nflows];
   }
  catch(exception& e)
  { cout << "Standard exception: " << e.what() << endl; }
	
  try {
   Links = new DCR::DCRLink[nlinks];
   }
  catch(exception& e)
  { cout << "Standard exception: " << e.what() << endl; }	

  FNSSgetLinks();	
  FNSSgetFlows();

  fname = filename;
  format = ftype;	
  }

/*--------------------------------------------------------------------------*/
/*------------------------------ GENERATION --------------------------------*/
/*--------------------------------------------------------------------------*/
  
void DCRwriter::Generate()
{
 if(format == 's') { write2MulgenFormat(); return; }
 if(format == 'm') { write2MnetgenFormat(); return; }
 cout << "\n Sorry, unknown format" << endl;
 }

/*--------------------------------------------------------------------------*/
/*------------------------------ GETTERS -----------------------------------*/
/*--------------------------------------------------------------------------*/

int DCRwriter::getNumNodes()
{ return nnodes; }

/*--------------------------------------------------------------------------*/

int DCRwriter::getNumLinks()
{ return nlinks; }

/*--------------------------------------------------------------------------*/

int DCRwriter::getNumFlows()
{ return nflows; }


/*--------------------------------------------------------------------------*/
/*------------------------------ DESTRUCTOR --------------------------------*/
/*--------------------------------------------------------------------------*/

DCRwriter::~DCRwriter()
{
 delete [] Flows;
 delete [] Links;
 }

/*--------------------------------------------------------------------------*/
/*-------------- PRIVATE METHODS -------------------------------------------*/
/*--------------------------------------------------------------------------*/

void DCRwriter::write2MulgenFormat()
{
  int j, i;
  cout << "\nGenerate and write *mulgen* format file " << fname << "\n";

  ofstream outfile;
  outfile.open(fname);

  //#nodes, #arcs, #flows
  outfile << nnodes << "\t" << nlinks << "\t" << nflows;

  //\forall arcs
  //<from_node_id> <to_node_id> <fixed_cost> <mutual_cap>
  //<flow_id> <arc cost for flow> <individual cap>

  for(i = 0; i < nlinks; i++) {
    outfile << "\n" << Links[i].startnode +1 << "\t" << Links[i].endnode +1
	    << "\t" << 1 << "\t" << Links[i].mutualcap << "\t" << nflows;
    for(j = 0; j < nflows; j++)
      outfile << "\n" << j + 1 << "\t" << 1 << "\t" << Links[i].mutualcap;
    }
	
  for(i = 0; i < nflows; i++) {
    outfile << "\n" << i+1 << "\t" << Flows[i].sourcenode +1 << "\t"
	    << Flows[i].rate;
    outfile << "\n" << i+1 << "\t" << Flows[i].sinknode +1 << "\t"
	    << -Flows[i].rate;
   }
	
  outfile.close();
 }

/*--------------------------------------------------------------------------*/
	
void DCRwriter::write2MnetgenFormat()
{
  int i;
  int l = strlen( fname );
  char *FN = new char[l+5];
  strcpy( FN , fname ); 
  ofstream outfile;  

  //.nod file 
  strcpy( FN + l , ".nod" );

  outfile.open(FN);

  //#flows, #nodes, #arcs, #capacitated arcs
  outfile << nflows << "\t" << nnodes << "\t" << nlinks << "\t" << nlinks;

  outfile.close();

  //.arc file
  strcpy( FN + l , ".arc" );

  outfile.open(FN);

  //arc_name from_node to_node commodity cost capacity mutualcap_pointer

  for(i = 0; i < nlinks; i++)
    outfile << i + 1 << "\t" << Links[i].startnode + 1 << "\t"
	    << Links[i].endnode + 1 << "\t" << -1 << "\t" << 1 << "\t"
	    <<  Links[i].mutualcap << "\t" << i + 1<< endl;		

  outfile.close();

  //.sup file             
  strcpy( FN + l , ".sup" );

  outfile.open(FN);

  //node commodity supply
	
  for(i = 0; i < nflows; i++) {
    outfile << Flows[i].sourcenode + 1 << "\t" << i + 1 << "\t"
	    << Flows[i].rate << endl;
    outfile << Flows[i].sinknode + 1 << "\t" << i + 1 << "\t"
	    << - Flows[i].rate << endl;				
   }

  outfile.close();

  //.mut file             
  strcpy( FN + l , ".mut" );

  outfile.open(FN);

  //mutualcap_pointer
  for(i = 0; i < nlinks; i++)
    outfile << i + 1 << "\t" << Links[i].mutualcap << endl;

  outfile.close();

  delete [] FN;
 }

/*--------------------------------------------------------------------------*/

void DCRwriter::FNSSgetFlows()
{ 
  int k = 0;

  for(set<pair<string, string>>::iterator it = flows.begin() ;
      it != flows.end(); it++) {
   #ifdef FLOWSLIMIT
    if(k >= nflows) break;
   #endif
   Flows[k].sourcenode = atoi((*it).first.c_str()); 
   Flows[k].sinknode = atoi((*it).second.c_str()); 
   Flows[k].burst =  -1;  //must be generated
   Flows[k].rate = traffic_matrix.getFlow(*it).getValue()*1000;//Gbps 
   Flows[k].deadline = -1;  //must be generated
   k++;
   }
 }
 
/*--------------------------------------------------------------------------*/

void DCRwriter::FNSSgetLinks()
{
  int j = 0;

  fnss::Edge e;
  for(set<pair<string,string>>::iterator it = edges.begin() ;
      it != edges.end(); it++) {
    e = topology.getEdge(*it);
    Links[j].speed = Links[j].mutualcap = e.getCapacity().getValue()*1000;
    Links[j].delay = -1; //must be generated
    Links[j].startnode = atoi((*it).first.c_str());
    Links[j].endnode = atoi((*it).second.c_str());
    j++;

    if(!topology.isDirected()) {
     Links[j].speed = Links[j].mutualcap = e.getCapacity().getValue()*1000;
     Links[j].delay = -1; //must be generated
     Links[j].startnode = Links[j-1].endnode;
     Links[j].endnode = Links[j-1].startnode;
     j++;
     }
   }
 }

/*--------------------------------------------------------------------------*/
/*------------------------ End File DCRwriter.cpp --------------------------*/
/*--------------------------------------------------------------------------*/
